import { Doodler, Vector } from "@bearmetal/doodler";
import { Train, TrainCar } from "./train.ts";
import { getContextItem } from "../lib/context.ts";
import { ResourceManager } from "../lib/resources.ts";
import { debug } from "node:console";
import { averageAngles, lerpAngle } from "../math/lerp.ts";

export class LargeLady extends TrainCar {
  scale = 1;
  constructor() {
    const resources = getContextItem<ResourceManager>("resources");
    const img = resources.get<HTMLImageElement>("snr:sprite/LargeLady")!;
    super(50, 10, img, 132, 23, {
      at: new Vector(0, 0),
      width: 132,
      height: 23,
    });

    this.bogies = [
      {
        pos: new Vector(0, 0),
        angle: 0,
        length: 35 * this.scale,
        sprite: {
          at: new Vector(0, 24),
          width: 35,
          height: 23,
          offset: new Vector(-23, -11.5),
        },
      },
      {
        pos: new Vector(0, 0),
        angle: 0,
        length: 64 * this.scale,
        // sprite: {
        //   at: new Vector(0, 23),
        //   width: 33,
        //   height: 19,
        //   offset: new Vector(-19, -9.5),
        // },
        sprite: {
          at: new Vector(36, 24),
          width: 60,
          height: 23,
          offset: new Vector(-35, -11.5),
        },
      },
      {
        pos: new Vector(0, 0),
        angle: 0,
        length: 35 * this.scale,
        sprite: {
          at: new Vector(36, 24),
          width: 60,
          height: 23,
          offset: new Vector(-35, -11.5),
        },
      },
      {
        pos: new Vector(0, 0),
        angle: 0,
        length: 28,
        sprite: {
          at: new Vector(97, 24),
          width: 22,
          height: 23,
          offset: new Vector(-11, -11.5),
        },
      },
    ];

    this.leading = 23;
  }

  drawAngle?: number;

  override draw(): void {
    const doodler = getContextItem<Doodler>("doodler");
    for (const b of this.bogies) {
      if (!b.sprite) continue;
      doodler.drawRotated(b.pos, b.angle + (b.rotate ? 0 : Math.PI), () => {
        doodler.drawSprite(
          this.img,
          b.sprite!.at,
          b.sprite!.width,
          b.sprite!.height,
          b.pos.copy().add(b.sprite!.offset ?? new Vector(0, 0)),
          b.sprite!.width,
          b.sprite!.height,
        );
      });
    }

    const b = this.bogies[2];
    const a = this.bogies[1];
    // const origin = Vector.add(Vector.sub(a.pos, b.pos).div(2), b.pos);
    // const angle = Vector.sub(b.pos, a.pos).heading();
    const debug = getContextItem<Debug>("debug");
    if (debug.bogies) return;

    const difAngle = Vector.sub(a.pos, b.pos).heading();
    if (this.drawAngle == undefined) this.drawAngle = b.angle + Math.PI;
    const origin = b.pos.copy().add(new Vector(33, 0).rotate(difAngle));
    const angle = b.angle;
    const avgAngle = averageAngles(difAngle, angle) + Math.PI;
    this.drawAngle = lerpAngle(this.drawAngle, avgAngle, .2);

    doodler.drawRotated(origin, this.drawAngle, () => {
      this.sprite
        ? doodler.drawSprite(
          this.img,
          this.sprite.at,
          this.sprite.width,
          this.sprite.height,
          origin.copy().sub(
            this.imgWidth * this.scale / 2,
            this.imgHeight * this.scale / 2,
          ),
          this.imgWidth * this.scale,
          this.imgHeight * this.scale,
        )
        : doodler.drawImage(
          this.img,
          origin.copy().sub(this.imgWidth / 2, this.imgHeight / 2),
        );
    });
    // doodler.drawCircle(origin, 4, { color: "blue" });

    doodler.deferDrawing(() => {
      doodler.drawRotated(origin, this.drawAngle! + Math.PI, () => {
        doodler.drawSprite(
          this.img,
          new Vector(133, 0),
          28,
          23,
          origin.copy().sub(93, this.imgHeight / 2),
          28,
          23,
        );
      });
    });
  }
}

export class LargeLadyTender extends TrainCar {
  constructor() {
    const resources = getContextItem<ResourceManager>("resources");
    const sprite = resources.get("snr:sprite/LargeLady");
    super(40, 39, sprite, 98, 23, {
      at: new Vector(0, 48),
      width: 98,
      height: 23,
    });

    this.leading = 19;
  }

  override draw(): void {
    const doodler = getContextItem<Doodler>("doodler");
    const b = this.bogies[0];
    doodler.drawRotated(b.pos, b.angle, () => {
      doodler.drawSprite(
        this.img,
        new Vector(97, 24),
        22,
        23,
        b.pos.copy().sub(11, 11.5),
        22,
        23,
      );
    });

    const angle = Vector.sub(this.bogies[1].pos, this.bogies[0].pos).heading();
    const origin = this.bogies[1].pos.copy().add(
      new Vector(-11, 0).rotate(angle),
    );
    doodler.drawRotated(origin, angle, () => {
      this.sprite
        ? doodler.drawSprite(
          this.img,
          this.sprite.at,
          this.sprite.width,
          this.sprite.height,
          origin.copy().sub(this.imgWidth / 2, this.imgHeight / 2),
          this.imgWidth,
          this.imgHeight,
        )
        : doodler.drawImage(
          this.img,
          origin.copy().sub(this.imgWidth / 2, this.imgHeight / 2),
        );
    });
  }
}
